==> autoads_test-0.1.3.tar.gz: autoads_test-0.1.3/examples/get_keywords_to_upload.py <==
import os
import uuid
import numpy as np
import pandas as pd
from google.ads.googleads.client import GoogleAdsClient
from sentence_transformers import SentenceTransformer
from autoads.models import get_similarity_matrix
from autoads.keywords import get_keywords_metrics,make_live_request,get_results
from autoads.gads import get_existing, get_search_term_report

email = 'weilbacherindustries@gmail.com' # email in data for seo
api_key = '05f014a493983975' # api key for data for seo
match_extract = ['exact', 'phrase']  # exact, phrase, broad
df_path = 'data/df_final.csv' # csv where keywords are stored
save_path = 'data' #all the csvs will be stored on this folder
path = '/home/maunish/Upwork Projects/Google-Ads-Project/examples/google-ads.yaml'
customer_id = '6554081276' # google ads customer id
first_threshold = 0.8 # similarity threshold for keyword similarity
second_threshold = 0.8 # similarity threshold for description similarity
conversion_threshold = 1 #  get conversion >= conversion_threshold
allowed_funnels = ['LF','MF'] # available options LF, MF, UF
similarity_model_path = 'Maunish/ecomm-sbert' # this model is already uploaded on huggingface so no need to download

os.makedirs(save_path,exist_ok=True)
googleads_client = GoogleAdsClient.load_from_storage(path=path, version="v9")

df = pd.read_csv(df_path)

print("Getting existing keywords")
df_existing = get_existing(googleads_client,customer_id)
# df_existing = pd.read_csv(save_path+'/df_existing.csv')
keywords_to_remove = df_existing[df_existing['camp_status'].isin(['ENABLED'])]['keyword_name'].unique().tolist()
df = df[~df["Keywords"].isin(keywords_to_remove)] # if campaign in enabled and keyword clash remove it.

#First pass of filters
print("Applying filters on the data")
df = df[~df["Keywords"].isin(keywords_to_remove)]
df = df[df['similarity'] >= first_threshold]
df = df[df['lf/mf/uf'].isin(allowed_funnels)].reset_index(drop=True)
df = df.drop_duplicates("Keywords")

#Second pass of similarity with page titles
tasks = [
    'organic',
    # 'people_also_ask',
]
task_params = {
    'organic': ['description'],
    # 'people_also_ask': ['title'],
}

print("Getting meta titles from SERP")
seed_keywords = df["Keywords2"].unique().tolist()
for keyword in seed_keywords:
    print(keyword)
    response,_ = make_live_request(keyword,email,api_key)
    results = get_results(response,tasks,task_params)
    if results.get('organic_description') and response['tasks'][0]['result']:
        organic_description = str(results['organic_description'])
        organic_description = organic_description.replace("[","").replace("]","")
        df.loc[df["Keywords2"]== keyword,'seed_organic_description'] = str(organic_description)
    else:
        df.loc[df["Keywords2"]== keyword,'seed_organic_description'] =  keyword

all_keywords = df["Keywords"].unique().tolist()
for keyword in all_keywords:
    print(keyword)
    response,_ = make_live_request(keyword,email,api_key)
    results = get_results(response,tasks,task_params)
    if results.get('organic_description') and response['tasks'][0]['result']:
        organic_description = str(results['organic_description'])
        organic_description = organic_description.replace("[","").replace("]","")
        df.loc[df["Keywords"]== keyword,'organic_description'] = organic_description
    else:
         df.loc[df["Keywords"]== keyword,'organic_description'] = [keyword]  

model = SentenceTransformer(similarity_model_path)

print("Performing similarity using organic description")
for i,row in df.iterrows():
    all_titles = row['seed_organic_description'].split(',')
    all_seed_titles = row['organic_description'].split(',')
    similarity_matrix = get_similarity_matrix(model,all_titles,all_seed_titles)
    df.loc[i,'second_similarity'] = similarity_matrix.max(axis=1).mean()

df = df[df['second_similarity'] >= second_threshold]

df_enabled = df_existing[df_existing['camp_status'] == 'ENABLED']
df_paused = df_existing[df_existing['camp_status'] == 'PAUSED']

if len(df) != 0:
    if len(df_enabled) == 0:
        print("No adgroups to expand to")
    else:
        print("Finding similarity with existing keywords")
        existing_campaign_id = df_enabled['camp_id'].tolist()
        existing_campaign_status = df_enabled['camp_status'].tolist()
        existing_keywords = df_enabled['keyword_name'].tolist()
        new_keywords = df['Keywords'].tolist()
        model = SentenceTransformer(similarity_model_path)
        similarity_matrix = get_similarity_matrix(model,existing_keywords,new_keywords)
        similarity_indexes = np.where(similarity_matrix>=first_threshold)

        if len(similarity_indexes[0]) != 0:
            keyword_campaign_id_pair = list()
            for i,j in zip(similarity_indexes[0],similarity_indexes[1]):
                keyword_campaign_id_pair.append((new_keywords[i],
                                                existing_campaign_id[j],
                                                existing_campaign_status[j],
                                                similarity_matrix[i][j]))

            df_expand = pd.DataFrame(keyword_campaign_id_pair,columns=['Keywords','camp_id','camp_status','similarity'])
            df_expand = df_expand.loc[df_expand.groupby(['Keywords'])['similarity'].idxmax()].reset_index(drop=True)
            df_expand = df_expand.loc[df_expand['similarity']!=1]
            if len(df_expand) != 0:
                df_expand.to_csv(save_path+'/df_expand.csv',index=False)
                df_expand = df_expand.drop('similarity',axis=1)
                df = df.merge(df_expand,how='left',on='Keywords')
            else:
                print("All the matching keywords are already present in the ad group")
        else:
            print("No matching campaigns found for expanding")

        print("Finding keywords to add in already existing campaign")
        already_exist_campaign = df_enabled['camp_name'].tolist()
        df.loc[df["Keywords2"].isin(df_enabled['camp_name'].tolist()),'camp_id'] = df_enabled.loc[df_enabled["camp_name"].isin(df['Keywords2'].tolist()),'camp_id']

    # code for altering campaign names
    paused_campaigns = df_paused['camp_name'].unique().tolist()
    for i,row in df[df['camp_id'].isna()].iterrows():
        if row['Keywords2'] in paused_campaigns:
            df.loc[i,'Keywords2'] = row['Keywords2'] + "_" + str(uuid.uuid4())

print("Getting search term report")
df_search_term_report = get_search_term_report(googleads_client,customer_id)
# df_search_term_report = pd.read_csv('data/search_term_df.csv')
df_search_term_report = df_search_term_report.loc[df_search_term_report['metrics_conversions']>=conversion_threshold,['stv_search_term','adgroup_camp']]
df_search_term_report['Keywords'] = df_search_term_report['stv_search_term']
df_search_term_report = df_search_term_report[~df_search_term_report["Keywords"].isin(keywords_to_remove)]
df_search_term_report['camp_id'] = df_search_term_report['adgroup_camp'].apply(lambda x: x.split('/')[-1])
df_search_term_report.drop(['stv_search_term','adgroup_camp'],axis=1,inplace=True)
df = pd.concat([df,df_search_term_report],axis=0,ignore_index=True)
df = df.drop_duplicates('Keywords')

if len(df) == 0:
    print("No new keywords to add")
else:
    print("Getting Keywords metrics")
    df = get_keywords_metrics(email,api_key,df,match_extract)

df_search_term_report.to_csv(save_path+'/df_search_term_report.csv',index=False)
df_existing.to_csv(save_path+'/df_existing.csv',index=False)
df.to_csv(save_path+'/keywords_to_upload.csv',index=False)

==> autoads_test-0.1.4.tar.gz: autoads_test-0.1.4/examples/get_keywords_to_upload.py <==
import os
import uuid
import numpy as np
import pandas as pd
from google.ads.googleads.client import GoogleAdsClient
from sentence_transformers import SentenceTransformer
from autoads.models import get_similarity_matrix
from autoads.keywords import get_keywords_metrics,make_live_request,get_results
from autoads.gads import get_existing

email = 'weilbacherindustries@gmail.com' # email in data for seo
api_key = '05f014a493983975' # api key for data for seo
match_extract = ['exact', 'phrase']  # exact, phrase, broad
df_path = 'data/df_final.csv' # csv where keywords are stored
save_path = 'data' #all the csvs will be stored on this folder
path = 'google-ads.yaml'
customer_id = '8306215642' #google ads customer id
first_threshold = 0.8 # similarity threshold for keyword similarity
second_threshold = 0.8 # similarity threshold for description similarity
conversion_threshold = 1 #  get conversion >= conversion_threshold
allowed_funnels = ['LF','MF'] # available options LF, MF, UF
similarity_model_path = 'Maunish/ecomm-sbert' # this model is already uploaded on huggingface so no need to download

os.makedirs(save_path,exist_ok=True)
googleads_client = GoogleAdsClient.load_from_storage(path=path, version="v9")

df = pd.read_csv(df_path)

print("Getting existing keywords")
df_existing = get_existing(googleads_client,customer_id)
# df_existing = pd.read_csv(save_path+'/df_existing.csv')
keywords_to_remove = df_existing[df_existing['camp_status'].isin(['ENABLED'])]['keyword_name'].unique().tolist()
df = df[~df["Keywords"].isin(keywords_to_remove)] # if campaign in enabled and keyword clash remove it.

#First pass of filters
print("Applying filters on the data")
df = df[~df["Keywords"].isin(keywords_to_remove)]
df = df[df['similarity'] >= first_threshold]
df = df[df['lf/mf/uf'].isin(allowed_funnels)].reset_index(drop=True)
df = df.drop_duplicates("Keywords")

#Second pass of similarity with page titles
tasks = [
    'organic',
    # 'people_also_ask',
]
task_params = {
    'organic': ['description'],
    # 'people_also_ask': ['title'],
}

print("Getting meta titles from SERP")
seed_keywords = df["Keywords2"].unique().tolist()
for keyword in seed_keywords:
    print(keyword)
    response,_ = make_live_request(keyword,email,api_key)
    results = get_results(response,tasks,task_params)
    if results.get('organic_description') and response['tasks'][0]['result']:
        organic_description = str(results['organic_description'])
        organic_description = organic_description.replace("[","").replace("]","")
        df.loc[df["Keywords2"]== keyword,'seed_organic_description'] = str(organic_description)
    else:
        df.loc[df["Keywords2"]== keyword,'seed_organic_description'] =  keyword

all_keywords = df["Keywords"].unique().tolist()
for keyword in all_keywords:
    print(keyword)
    response,_ = make_live_request(keyword,email,api_key)
    results = get_results(response,tasks,task_params)
    if results.get('organic_description') and response['tasks'][0]['result']:
        organic_description = str(results['organic_description'])
        organic_description = organic_description.replace("[","").replace("]","")
        df.loc[df["Keywords"]== keyword,'organic_description'] = organic_description
    else:
         df.loc[df["Keywords"]== keyword,'organic_description'] = [keyword]  

model = SentenceTransformer(similarity_model_path)

print("Performing similarity using organic description")
for i,row in df.iterrows():
    all_titles = row['seed_organic_description'].split(',')
    all_seed_titles = row['organic_description'].split(',')
    similarity_matrix = get_similarity_matrix(model,all_titles,all_seed_titles)
    df.loc[i,'second_similarity'] = similarity_matrix.max(axis=1).mean()

df = df[df['second_similarity'] >= second_threshold]

df_enabled = df_existing[df_existing['camp_status'] == 'ENABLED']
df_paused = df_existing[df_existing['camp_status'] == 'PAUSED']

if len(df) != 0:
    if len(df_enabled) == 0:
        print("No adgroups to expand to")
    else:
        print("Finding similarity with existing keywords")
        existing_campaign_id = df_enabled['camp_id'].tolist()
        existing_campaign_status = df_enabled['camp_status'].tolist()
        existing_keywords = df_enabled['keyword_name'].tolist()
        new_keywords = df['Keywords'].tolist()
        model = SentenceTransformer(similarity_model_path)
        similarity_matrix = get_similarity_matrix(model,existing_keywords,new_keywords)
        similarity_indexes = np.where(similarity_matrix>=first_threshold)

        if len(similarity_indexes[0]) != 0:
            keyword_campaign_id_pair = list()
            for i,j in zip(similarity_indexes[0],similarity_indexes[1]):
                keyword_campaign_id_pair.append((new_keywords[i],
                                                existing_campaign_id[j],
                                                existing_campaign_status[j],
                                                similarity_matrix[i][j]))

            df_expand = pd.DataFrame(keyword_campaign_id_pair,columns=['Keywords','camp_id','camp_status','similarity'])
            df_expand = df_expand.loc[df_expand.groupby(['Keywords'])['similarity'].idxmax()].reset_index(drop=True)
            df_expand = df_expand.loc[df_expand['similarity']!=1]
            if len(df_expand) != 0:
                df_expand.to_csv(save_path+'/df_expand.csv',index=False)
                df_expand = df_expand.drop('similarity',axis=1)
                df = df.merge(df_expand,how='left',on='Keywords')
            else:
                print("All the matching keywords are already present in the ad group")
        else:
            print("No matching campaigns found for expanding")

        print("Finding keywords to add in already existing campaign")
        already_exist_campaign = df_enabled['camp_name'].tolist()
        df.loc[df["Keywords2"].isin(df_enabled['camp_name'].tolist()),'camp_id'] = df_enabled.loc[df_enabled["camp_name"].isin(df['Keywords2'].tolist()),'camp_id']

    # code for altering campaign names
    paused_campaigns = df_paused['camp_name'].unique().tolist()
    for i,row in df[df['camp_id'].isna()].iterrows():
        if row['Keywords2'] in paused_campaigns:
            df.loc[i,'Keywords2'] = row['Keywords2'] + "_" + str(uuid.uuid4())

df = df.drop_duplicates('Keywords')

if len(df) == 0:
    print("No new keywords to add")
else:
    print("Getting Keywords metrics")
    # df = get_keywords_metrics(email,api_key,df,match_extract)

df_existing.to_csv(save_path+'/df_existing.csv',index=False)
df.to_csv(save_path+'/keywords_to_upload.csv',index=False)
